/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/03/01
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy;

import org.jiemamy.model.ConnectionProfile;
import org.jiemamy.model.DiagramPresentationModel;
import org.jiemamy.model.JiemamyElement;
import org.jiemamy.model.NodeProfile;
import org.jiemamy.model.node.StickyModel;

/**
 * J-view内の各モデルの「概念的な」（実装としては強要しない）プロパティを表す列挙型を集約したクラス。
 * 
 * @param <T> モデルの型
 * @since 0.2
 * @author shin1ogawa
 */
public interface JiemamyViewProperty<T extends JiemamyElement> extends JiemamyProperty<T> {
	
	/**
	 * {@link ConnectionProfile}のプロパティを表す列挙型。
	 * 
	 * @since 0.2
	 * @author shin1ogawa
	 */
	public enum ConnectionProfileProperty implements JiemamyViewProperty<ConnectionProfile> {
		
		/** {@link ConnectionProfile#getBendpoints()}で取得できるプロパティ */
		bendpoints,

		/** {@link ConnectionProfile#getColor()}で取得できるプロパティ */
		color
	}
	
	/**
	 * {@link DiagramPresentationModel}のプロパティを表す列挙型。
	 * 
	 * @since 0.2
	 * @author shin1ogawa
	 */
	public enum DiagramPresentationProperty implements JiemamyViewProperty<DiagramPresentationModel> {
		
		/** {@link DiagramPresentationModel#getConnectionProfiles()}で取得できるプロパティ */
		connectionProfiles,

		/** {@link DiagramPresentationModel#getNodeProfiles()}で取得できるプロパティ */
		nodeProfiles,

		/** {@link DiagramPresentationModel#getLevel()}で取得できるプロパティ */
		level,

		/** {@link DiagramPresentationModel#getMode()}で取得できるプロパティ */
		mode,

		/** {@link DiagramPresentationModel#getName()}で取得できるプロパティ */
		name
	}
	
	/**
	 * {@link NodeProfile}のプロパティを表す列挙型。
	 * 
	 * @since 0.2
	 * @author shin1ogawa
	 */
	public enum NodeProfileProperty implements JiemamyViewProperty<NodeProfile> {
		
		/** {@link NodeProfile#getBoundary()}で取得できるプロパティ */
		boundary,

		/** {@link NodeProfile#getColor()}で取得できるプロパティ */
		color
	}
	
	/**
	 * {@link StickyModel}のプロパティを表す列挙型。
	 * 
	 * @since 0.2
	 * @author daisuke
	 */
	public enum StickyProperty implements JiemamyViewProperty<StickyModel> {
		
		/** {@link StickyModel#getContents()}で取得できるプロパティ */
		contents
	}
}
