/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/03/25
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.composer.importer;

import java.util.Map;

import org.jiemamy.JiemamyFactory;
import org.jiemamy.Migration;
import org.jiemamy.composer.ImportException;
import org.jiemamy.model.ConnectionProfile;
import org.jiemamy.model.DiagramPresentationModel;
import org.jiemamy.model.DiagramPresentations;
import org.jiemamy.model.NodeProfile;
import org.jiemamy.model.RootModel;
import org.jiemamy.model.attribute.constraint.ForeignKey;
import org.jiemamy.model.connection.ConnectionAdapter;
import org.jiemamy.model.entity.EntityModel;
import org.jiemamy.model.geometory.JmRectangle;
import org.jiemamy.model.node.NodeAdapter;

/**
 * インポートと共に、インポートしたエンティティにビジュアル情報を設定するインポータ。
 * 
 * @author daisuke
 */
public class DatabaseImporterWithView extends DatabaseImporter {
	
	@Override
	public boolean importModel(RootModel rootModel, DatabaseImportConfig config) throws ImportException {
		boolean result = super.importModel(rootModel, config);
		
		JiemamyFactory factory = rootModel.getJiemamy().getFactory();
		DiagramPresentations presentations = rootModel.getAdapter(DiagramPresentations.class);
		DiagramPresentationModel presentationModel = presentations.get(Migration.DIAGRAM_INDEX);
		Map<NodeAdapter, NodeProfile> nodeProfiles = presentationModel.getNodeProfiles();
		Map<ConnectionAdapter, ConnectionProfile> connectionProfiles = presentationModel.getConnectionProfiles();
		
		for (EntityModel entityModel : getImportedEntities()) {
			NodeProfile nodeProfile = factory.newModel(NodeProfile.class);
			nodeProfile.setBoundary(new JmRectangle(0, 0));
			nodeProfiles.put(entityModel.getAdapter(NodeAdapter.class), nodeProfile);
		}
		for (ForeignKey foreignKey : getImportedForeignKeys()) {
			ConnectionProfile connectionProfile = factory.newModel(ConnectionProfile.class);
			connectionProfiles.put(foreignKey.getAdapter(ConnectionAdapter.class), connectionProfile);
		}
		return result;
	}
}
