/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2008/09/10
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.dialect;

import java.sql.Connection;

/**
 * DBメタデータの読み出し前後に実行する処理を司るインターフェイス。
 * 
 * @since 0.2
 * @author daisuke
 */
public interface ConnectionAdvisor {
	
	/**
	 * エンティティ・リレーションの読み出し前に実行する処理を行う。
	 * 
	 * @param connection DB接続
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 * @since 0.2
	 */
	void setUpRead(Connection connection);
	
	/**
	 * エンティティ・リレーションの読み出し後に実行する処理を行う。
	 * 
	 * @param connection DB接続
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 * @since 0.2
	 */
	void tearDownRead(Connection connection);
}
