/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2008/07/07
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.dialect.generic;

import java.util.List;

import org.apache.commons.collections15.list.UnmodifiableList;

import org.jiemamy.dialect.BuiltinDataTypeMold;
import org.jiemamy.dialect.DataTypeResolver;
import org.jiemamy.dialect.EmitConfig;
import org.jiemamy.dialect.internal.AbstractDialect;
import org.jiemamy.dialect.internal.BuiltinDataTypeMoldImpl;
import org.jiemamy.dialect.internal.impl.DefaultDataTypeResolver;
import org.jiemamy.dialect.internal.impl.DefaultDialectInitializer;
import org.jiemamy.dialect.internal.impl.DefaultMoldManager;
import org.jiemamy.model.RootModel;
import org.jiemamy.model.datatype.DataTypeCategory;
import org.jiemamy.model.datatype.adapter.PrecisionedDataTypeAdapter;
import org.jiemamy.model.datatype.adapter.SizedDataTypeAdapter;
import org.jiemamy.model.datatype.adapter.TimezonedDataTypeAdapter;
import org.jiemamy.model.sql.SqlStatement;
import org.jiemamy.utils.CollectionsUtil;

/**
 * 全てのSQL方言に対応し得る、最大公約数的なSQL方言実装クラス。
 * 
 * @author daisuke
 */
public final class GenericDialect extends AbstractDialect {
	
	private static List<BuiltinDataTypeMold> typeList;
	
	static {
		List<BuiltinDataTypeMold> list = CollectionsUtil.newArrayList();
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.INTEGER));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.SMALLINT));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.NUMERIC, PrecisionedDataTypeAdapter.class));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.DECIMAL, PrecisionedDataTypeAdapter.class));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.FLOAT, PrecisionedDataTypeAdapter.class));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.REAL, PrecisionedDataTypeAdapter.class));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.DOUBLE, PrecisionedDataTypeAdapter.class));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.BIT, SizedDataTypeAdapter.class));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.VARBIT, SizedDataTypeAdapter.class));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.BOOLEAN));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.CHARACTER, SizedDataTypeAdapter.class));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.VARCHAR, SizedDataTypeAdapter.class));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.CLOB, SizedDataTypeAdapter.class));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.BLOB, SizedDataTypeAdapter.class));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.DATE));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.TIME, TimezonedDataTypeAdapter.class));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.TIMESTAMP, TimezonedDataTypeAdapter.class));
		list.add(new BuiltinDataTypeMoldImpl(DataTypeCategory.INTERVAL));
		typeList = UnmodifiableList.decorate(list);
	}
	

	/**
	 * インスタンスを生成する。
	 */
	public GenericDialect() {
		super(new DefaultDialectInitializer(new DefaultMoldManager(typeList)) {
			
			private DefaultDataTypeResolver defaultDataTypeResolver;
			

			@Override
			public DataTypeResolver getDataTypeResolver() {
				if (defaultDataTypeResolver == null) {
					defaultDataTypeResolver = new DefaultDataTypeResolver(getMoldManager());
				}
				return defaultDataTypeResolver;
			}
		});
	}
	
	@Override
	public List<SqlStatement> emitStatements(RootModel rootModel, EmitConfig config) {
		throw new UnsupportedOperationException();
	}
	
	public String getName() {
		return "Generic Database";
	}
	
	@Override
	public String toString() {
		return this.getClass().getName();
	}
	
}
