/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/02/17
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.dialect.internal;

import java.util.Arrays;
import java.util.Collection;

import org.apache.commons.lang.Validate;

import org.jiemamy.dialect.BuiltinDataTypeMold;
import org.jiemamy.model.datatype.BuiltinDataType;
import org.jiemamy.model.datatype.DataTypeCategory;

/**
 * {@link BuiltinDataType}インスタンスを生成する鋳型情報の実装クラス。
 * 
 * @author daisuke
 */
public class BuiltinDataTypeMoldImpl implements BuiltinDataTypeMold {
	
	private final DataTypeCategory category;
	
	private final String name;
	
	private final Class<?>[] supportedAdapterClasses;
	

	/**
	 * インスタンスを生成する。
	 * 
	 * <p>型名には型カテゴリの{@link Enum#name()}結果が使用される。</p>
	 * 
	 * @param category 型カテゴリ
	 * @param supportedAdapterClasses この型がサポートするアダプタクラス
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public BuiltinDataTypeMoldImpl(DataTypeCategory category, Class<?>... supportedAdapterClasses) {
		this(category, category.name(), supportedAdapterClasses);
	}
	
	/**
	 * インスタンスを生成する。
	 * 
	 * @param category 型カテゴリ
	 * @param name 型名
	 * @param supportedAdapterClasses この型がサポートするアダプタクラス
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public BuiltinDataTypeMoldImpl(DataTypeCategory category, String name, Class<?>... supportedAdapterClasses) {
		Validate.notNull(category);
		Validate.notNull(name);
		Validate.notNull(supportedAdapterClasses);
		this.category = category;
		this.name = name;
		this.supportedAdapterClasses = supportedAdapterClasses;
	}
	
	public DataTypeCategory getCategory() {
		return category;
	}
	
	public String getName() {
		return name;
	}
	
	public Collection<Class<?>> getSupportedAdapterClasses() {
		return Arrays.asList(supportedAdapterClasses);
	}
	
}
