/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/03/31
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.dialect.internal.impl;

import java.util.List;

import org.apache.commons.lang.Validate;

import org.jiemamy.dialect.BuiltinDataTypeMold;
import org.jiemamy.dialect.MoldManager;
import org.jiemamy.model.datatype.BuiltinDataType;
import org.jiemamy.model.datatype.DataTypeCategory;

/**
 * {@link MoldManager}のデフォルト実装クラス。
 * 
 * @author daisuke
 */
public class DefaultMoldManager implements MoldManager {
	
	private final List<BuiltinDataTypeMold> typeList;
	

	/**
	 * インスタンスを生成する。
	 * 
	 * @param typeList データ型鋳型のリスト
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public DefaultMoldManager(List<BuiltinDataTypeMold> typeList) {
		Validate.notNull(typeList);
		this.typeList = typeList;
	}
	
	public BuiltinDataTypeMold findDataTypeMold(BuiltinDataType builtinDataType) {
		return findDataTypeMold(builtinDataType.getCategory(), builtinDataType.getTypeName());
	}
	
	public BuiltinDataTypeMold findDataTypeMold(DataTypeCategory category) {
		Validate.notNull(category);
		return findDataTypeMold(category, category.name());
	}
	
	public BuiltinDataTypeMold findDataTypeMold(DataTypeCategory category, String typeName) {
		Validate.notNull(typeName);
		for (BuiltinDataTypeMold mold : typeList) {
			if (mold.getCategory() == category && mold.getName().equals(typeName)) {
				return mold;
			}
		}
		for (BuiltinDataTypeMold mold : typeList) {
			if (mold.getName().equals(typeName)) {
				return mold;
			}
		}
		for (BuiltinDataTypeMold mold : typeList) {
			if (mold.getCategory() == category) {
				return mold;
			}
		}
		return BuiltinDataTypeMold.UNKNOWN;
	}
	
	public List<BuiltinDataTypeMold> getTypeList() {
		return typeList;
	}
	
}
