/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2008/08/30
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.exception;

import org.jiemamy.dialect.Dialect;

/**
 * SQL方言の実装が不正である時にスローされる例外。
 * 
 * @since 0.2
 * @author daisuke
 */
@SuppressWarnings("serial")
public class IllegalDialectImplementationException extends IllegalImplementationException {
	
	/** 不正実装と判定されたSQL方言クラス */
	private Class<? extends Dialect> dialectClass;
	

	/**
	 * インスタンスを生成する。
	 * 
	 * @param dialectClass 不正実装のSQL方言クラス
	 * @param message 例外メッセージ
	 * @since 0.2
	 */
	public IllegalDialectImplementationException(Class<? extends Dialect> dialectClass, String message) {
		super(message);
		this.dialectClass = dialectClass;
	}
	
	/**
	 * インスタンスを生成する。
	 * 
	 * @param dialectClass 不正実装のSQL方言クラス
	 * @param cause 起因例外
	 * @since 0.2
	 */
	public IllegalDialectImplementationException(Class<? extends Dialect> dialectClass, Throwable cause) {
		super(cause);
		this.dialectClass = dialectClass;
	}
	
	/**
	 * インスタンスを生成する。
	 * 
	 * @param message 例外メッセージ
	 * @since 0.2
	 */
	public IllegalDialectImplementationException(String message) {
		super(message);
	}
	
	/**
	 * 不正実装と判定されたSQL方言クラスを取得する。
	 * 
	 * @return 不正実装と判定されたSQL方言クラス
	 * @since 0.2
	 */
	public Class<? extends Dialect> getDialectClass() {
		return dialectClass;
	}
	
}
