/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2008/12/01
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.exception;

/**
 * 不正実装と判断された場合。
 * 
 * @since 0.2
 * @author daisuke
 */
@SuppressWarnings("serial")
public class IllegalImplementationException extends JiemamyRuntimeException {
	
	/** 不正実装と判定されたクラスを表すオブジェクト */
	private Object target;
	

	/**
	 * インスタンスを生成する。
	 * 
	 * @param target 不正実装だと判断されたオブジェクト
	 * @since 0.2
	 */
	public IllegalImplementationException(Object target) {
		super();
		this.target = target;
	}
	
	/**
	 * インスタンスを生成する。
	 * 
	 * @param target 不正実装だと判断されたオブジェクト
	 * @param message 例外メッセージ
	 * @since 0.2
	 */
	public IllegalImplementationException(Object target, String message) {
		super(message);
		this.target = target;
	}
	
	/**
	 * インスタンスを生成する。
	 * 
	 * @param target 不正実装だと判断されたオブジェクト
	 * @param cause 起因例外
	 * @since 0.2
	 */
	public IllegalImplementationException(Object target, Throwable cause) {
		super(cause);
	}
	
	/**
	 * インスタンスを生成する。
	 * 
	 * @param message 例外メッセージ（不正実装だと判断されたオブジェクトを特定するための文字列）
	 * @since 0.2
	 */
	public IllegalImplementationException(String message) {
		super(message);
	}
	
	/**
	 * インスタンスを生成する。
	 * 
	 * @param message 例外メッセージ（不正実装だと判断されたオブジェクトを特定するための文字列）
	 * @param cause 起因例外
	 * @since 0.2
	 */
	public IllegalImplementationException(String message, Throwable cause) {
		super(message, cause);
	}
	
	/**
	 * 不正実装だと判断されたオブジェクト表すオブジェクトを取得する。
	 * 
	 * @return 不正実装だと判断されたオブジェクトを表すオブジェクト
	 * @since 0.2
	 */
	public Object getTarget() {
		return target;
	}
	
}
