/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/03/04
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.experimental.builder;

import org.jiemamy.exception.ElementNotFoundException;
import org.jiemamy.exception.ModelConsistenceException;
import org.jiemamy.model.attribute.ColumnModel;
import org.jiemamy.model.attribute.ColumnRef;
import org.jiemamy.model.attribute.constraint.ForeignKey;

/**
 * {@link ForeignKey}を簡単に構築するためのビルダインターフェイス。
 * 
 * @author daisuke
 */
public interface ForeignKeyBuilder extends JiemamyBuilder<ForeignKey> {
	
	/**
	 * カラム同士のマッピングを設定する。
	 * 
	 * @param keyColumn 参照元テーブルの「制約を受けるカラム」の一つ
	 * @param referenceColumn 参照先テーブルの主キーカラムの一つ
	 * @return {@code this}
	 * @throws ElementNotFoundException referenceColumnが、所属テーブルにおいてキー指定されていない場合
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 * @throws ModelConsistenceException foreignKeyColumnがconstrainedTableに所属していない場合
	 * @throws ModelConsistenceException keyColumnがreferenceTableに所属していない場合
	 * @since 0.2
	 */
	ForeignKeyBuilder mapping(ColumnModel keyColumn, ColumnModel referenceColumn);
	
	/**
	 * カラム同士のマッピングを設定する。
	 * 
	 * @param keyColumn 参照元テーブルの「制約を受けるカラム」の一つ
	 * @param referenceColumn 参照先テーブルの主キーカラムの一つ
	 * @return {@code this}
	 * @throws ElementNotFoundException referenceColumnが、所属テーブルにおいて主キー指定されていない場合
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 * @since 0.2
	 */
	ForeignKeyBuilder mapping(ColumnRef keyColumn, ColumnRef referenceColumn);
	
}
