/*
 * Copyright 2007-2009 MIYAMOTO Daisuke, jiemamy.org and the Others.
 * Created on 2009/01/01
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.experimental.builder;

import org.jiemamy.model.attribute.ColumnModel;
import org.jiemamy.model.entity.TableModel;

/**
 * {@link TableModel}を簡単に構築するためのビルダインターフェイス。
 * 
 * @author daisuke
 */
public interface TableBuilder extends JiemamyBuilder<TableModel> {
	
	/**
	 * カラムを追加する。
	 * 
	 * @param columnModel 追加するカラム
	 * @return {@code this}
	 * @since 0.2
	 */
	TableBuilder addColumn(ColumnModel columnModel);
	
	/**
	 * 主キーをカラムのインデックスで指定する。
	 * 
	 * @param index カラムのインデックス
	 * @param otherIndexes 2つ目以降のカラムのインデックス
	 * @return this
	 * @throws IndexOutOfBoundsException 指定したインデックスのカラムが存在しない場合
	 * @since 0.2
	 */
	TableBuilder primaryKey(int index, int... otherIndexes);
	
	/**
	 * テーブル名を設定する。
	 * 
	 * @param name テーブル名
	 * @return {@code this}
	 * @since 0.2
	 */
	TableBuilder setName(String name);
	
}
