/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2008/12/24
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.builder;

import org.apache.commons.lang.Validate;

import org.jiemamy.JiemamyFactory;
import org.jiemamy.experimental.builder.ViewBuilder;
import org.jiemamy.model.entity.ViewModel;

/**
 * {@link ViewModel}を構築するためのビルダー実装クラス。
 * 
 * @author daisuke
 */
public class ViewBuilderImpl extends AbstractJiemamyBuilder<ViewModel> implements ViewBuilder {
	
	/**
	 * インスタンスを生成する。
	 * 
	 * @param factory モデル生成のためのファクトリ
	 * @param name ビュー名
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public ViewBuilderImpl(JiemamyFactory factory, String name) {
		super(factory, ViewModel.class);
		Validate.notNull(name);
		model.setName(name);
	}
	
	public ViewBuilder definition(String definition) {
		model.setDefinition(definition);
		return this;
	}
	
	public ViewBuilder setName(String name) {
		model.setName(name);
		return this;
	}
	
	@Override
	boolean isComplete() {
		return model.getName() != null;
	}
}
