/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/01/18
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.editcommand;

import java.util.Collection;

import org.jiemamy.EventBroker;
import org.jiemamy.editcommand.Command;
import org.jiemamy.model.RootModel;
import org.jiemamy.model.datatype.DomainModel;

/**
 * {@link RootModel}に{@link DomainModel}を追加するEDITコマンド。
 * 
 * @see RemoveDomainFromRootCommand
 * @author shin1ogawa
 */
public class AddDomainToRootCommand extends AbstractAddToCollectionCommand<RootModel, DomainModel> {
	
	/**
	 * インスタンスを生成する。
	 * 
	 * @param eventBroker イベント通知用{@link EventBroker}
	 * @param rootModel 追加される{@link RootModel}
	 * @param domainModel 追加する{@link DomainModel}
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public AddDomainToRootCommand(EventBroker eventBroker, RootModel rootModel, DomainModel domainModel) {
		super(eventBroker, rootModel, domainModel);
	}
	
	/**
	 * インスタンスを生成する。
	 * 
	 * @param eventBroker イベント通知用{@link EventBroker}
	 * @param rootModel 追加される{@link RootModel}
	 * @param domainModel 追加する{@link DomainModel}
	 * @param index 追加位置をあらわすインデックス値
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public AddDomainToRootCommand(EventBroker eventBroker, RootModel rootModel, DomainModel domainModel, int index) {
		super(eventBroker, rootModel, domainModel, index);
	}
	
	public Command getNegateCommand() {
		return new RemoveDomainFromRootCommand(getEventBroker(), getTarget(), getElement());
	}
	
	@Override
	public Collection<? super DomainModel> getTargetCollection() {
		return getTarget().getDomains();
	}
}
