/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/04/13
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.editcommand;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.Validate;

import org.jiemamy.EventBroker;
import org.jiemamy.JiemamyFactory;
import org.jiemamy.editcommand.Command;
import org.jiemamy.model.JiemamyElement;
import org.jiemamy.model.dataset.DataSetModel;
import org.jiemamy.model.dataset.RecordModel;
import org.jiemamy.model.entity.TableModel;

/**
 * {@link DataSetModel}に、指定したテーブル用のレコード保存領域を確保するEDITコマンド。
 * 
 * @author daisuke
 */
public class AddRecordsToDataSetCommand extends AbstractCommand {
	
	/** レコード追加対象の{@link DataSetModel} */
	private final DataSetModel dataSetModel;
	
	/** レコード追加対象の{@link TableModel} */
	private final TableModel tableModel;
	
	/** データセット保存用リストとして使用するインスタンス */
	private final List<RecordModel> records;
	

	/**
	 * インスタンスを生成する。
	 * 
	 * @param eventBroker イベント通知用{@link EventBroker}
	 * @param dataSetModel レコード追加対象の{@link DataSetModel}
	 * @param tableModel レコード追加対象の{@link TableModel}
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public AddRecordsToDataSetCommand(EventBroker eventBroker, DataSetModel dataSetModel, TableModel tableModel) {
		this(eventBroker, dataSetModel, tableModel, new ArrayList<RecordModel>());
	}
	
	/**
	 * インスタンスを生成する。
	 * 
	 * @param eventBroker イベント通知用{@link EventBroker}
	 * @param dataSetModel レコード削除元の{@link DataSetModel}
	 * @param tableModel レコード削除対象の{@link TableModel}
	 * @param records データセット保存用リストとして使用するインスタンス
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public AddRecordsToDataSetCommand(EventBroker eventBroker, DataSetModel dataSetModel, TableModel tableModel,
			List<RecordModel> records) {
		super(eventBroker);
		Validate.notNull(dataSetModel);
		Validate.notNull(tableModel);
		Validate.notNull(records);
		this.dataSetModel = dataSetModel;
		this.tableModel = tableModel;
		this.records = records;
	}
	
	public Command getNegateCommand() {
		return new RemoveRecordsFromDataSetCommand(getEventBroker(), dataSetModel, tableModel);
	}
	
	public JiemamyElement getTarget() {
		return dataSetModel;
	}
	
	@Override
	protected void execute0() {
		if (records == null) {
			return;
		}
		JiemamyFactory factory = dataSetModel.getJiemamy().getFactory();
		dataSetModel.getRecords().put(factory.newReference(tableModel), records);
	}
	
}
