/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/03/02
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.editcommand;

import org.apache.commons.lang.Validate;

import org.jiemamy.EventBroker;
import org.jiemamy.editcommand.Command;
import org.jiemamy.model.ConnectionProfile;
import org.jiemamy.model.JiemamyElement;
import org.jiemamy.model.geometory.JmPoint;

/**
 * ベンドポイントを移動するEDITコマンド。
 * 
 * @author daisuke
 */
public class MoveBendpointCommand extends AbstractCommand {
	
	/** ダイアグラムエディタのインデックス（エディタ内のタブインデックス） */
	private final int diagramIndex;
	
	/** 編集対象コネクションのプロファイル */
	private final ConnectionProfile connectionProfile;
	
	/** ベンドポイントの位置インデックス */
	private final int bendpointIndex;
	
	/** 移動先の座標 */
	private final JmPoint newPoint;
	
	/** 移動元の座標 */
	private final JmPoint oldPoint;
	

	/**
	 * インスタンスを生成する。
	 * 
	 * @param eventBroker イベント通知用{@link EventBroker}
	 * @param diagramIndex ダイアグラムエディタのインデックス（エディタ内のタブインデックス）
	 * @param connectionProfile 編集対象コネクションのプロファイル
	 * @param bendpointIndex source側からtarget側に向かって数えたベンドポイントのインデックス
	 * @param newPoint 移動先の座標
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public MoveBendpointCommand(EventBroker eventBroker, int diagramIndex, ConnectionProfile connectionProfile,
			int bendpointIndex, JmPoint newPoint) {
		super(eventBroker);
		Validate.notNull(connectionProfile);
		Validate.notNull(newPoint);
		this.diagramIndex = diagramIndex;
		this.connectionProfile = connectionProfile;
		this.bendpointIndex = bendpointIndex;
		this.newPoint = newPoint;
		oldPoint = connectionProfile.getBendpoints().get(bendpointIndex);
	}
	
	public Command getNegateCommand() {
		return new MoveBendpointCommand(getEventBroker(), diagramIndex, connectionProfile, bendpointIndex, oldPoint);
	}
	
	public JiemamyElement getTarget() {
		return connectionProfile;
	}
	
	@Override
	protected void execute0() {
		connectionProfile.getBendpoints().set(bendpointIndex, newPoint);
	}
}
