/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/03/02
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.editcommand;

import java.util.Collection;

import org.jiemamy.EventBroker;
import org.jiemamy.editcommand.Command;
import org.jiemamy.model.ConnectionProfile;
import org.jiemamy.model.geometory.JmPoint;

/**
 * コネクションからベンドポイントを削除するEDITコマンド。
 * 
 * @author daisuke
 */
public class RemoveBendpointCommand extends AbstractRemoveFromCollectionCommand<ConnectionProfile, JmPoint> {
	
	/** ダイアグラムエディタのインデックス（エディタ内のタブインデックス） */
	private final int diagramIndex;
	
	private final ConnectionProfile connectionProfile;
	

	/**
	 * インスタンスを生成する。
	 * 
	 * @param eventBroker イベント通知用{@link EventBroker}
	 * @param diagramIndex ダイアグラムエディタのインデックス（エディタ内のタブインデックス）
	 * @param connectionProfile ベンドポイントが削除されるコネクションのビジュアル情報
	 * @param bendpointIndex source側からtarget側に向かって数えたベンドポイントのインデックス
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public RemoveBendpointCommand(EventBroker eventBroker, int diagramIndex, ConnectionProfile connectionProfile,
			int bendpointIndex) {
		super(eventBroker, connectionProfile, bendpointIndex);
		this.diagramIndex = diagramIndex;
		this.connectionProfile = connectionProfile;
		
		// HACK 本来基底クラスコンストラクタで初期化されるものだが、super()実行時にはconnectionProfileフィールドが初期化されておらず、getTargetCollection()が使用できない為、ここで初期化している。
		element = connectionProfile.getBendpoints().get(bendpointIndex);
	}
	
	/**
	 * インスタンスを生成する。
	 * 
	 * @param eventBroker イベント通知用{@link EventBroker}
	 * @param diagramIndex ダイアグラムエディタのインデックス（エディタ内のタブインデックス）
	 * @param connectionProfile ベンドポイントが削除されるコネクションのビジュアル情報
	 * @param point 削除対象の座標
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public RemoveBendpointCommand(EventBroker eventBroker, int diagramIndex, ConnectionProfile connectionProfile,
			JmPoint point) {
		super(eventBroker, connectionProfile, point);
		this.diagramIndex = diagramIndex;
		this.connectionProfile = connectionProfile;
	}
	
	public Command getNegateCommand() {
		if (getIndex() != -1) {
			return new AddBendpointCommand(getEventBroker(), diagramIndex, connectionProfile, -1, getElement());
		} else {
			return new AddBendpointCommand(getEventBroker(), diagramIndex, connectionProfile, getElement());
		}
	}
	
	@Override
	public Collection<? super JmPoint> getTargetCollection() {
		return connectionProfile.getBendpoints();
	}
	
}
