/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/01/18
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.editcommand;

import java.util.Collection;

import org.jiemamy.EventBroker;
import org.jiemamy.editcommand.Command;
import org.jiemamy.model.RootModel;
import org.jiemamy.model.dataset.DataSetModel;

/**
 * {@link RootModel}から{@link DataSetModel}を削除するEDITコマンド。
 * 
 * @see AddDataSetToRootCommand
 * @author shin1ogawa
 */
public class RemoveDataSetFromRootCommand extends AbstractRemoveFromCollectionCommand<RootModel, DataSetModel> {
	
	/**
	 * インスタンスを生成する。
	 * 
	 * @param eventBroker イベント通知用{@link EventBroker}
	 * @param rootModel 削除元の{@link RootModel}
	 * @param dataSetModel 削除される{@link DataSetModel}
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public RemoveDataSetFromRootCommand(EventBroker eventBroker, RootModel rootModel, DataSetModel dataSetModel) {
		super(eventBroker, rootModel, dataSetModel);
	}
	
	public Command getNegateCommand() {
		return new AddDataSetToRootCommand(getEventBroker(), getTarget(), getElement());
	}
	
	@Override
	public Collection<? super DataSetModel> getTargetCollection() {
		return getTarget().getDataSets();
	}
}
