/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/03/02
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.editcommand;

import java.util.List;

import org.apache.commons.lang.Validate;

import org.jiemamy.EventBroker;
import org.jiemamy.editcommand.Command;
import org.jiemamy.model.ConnectionProfile;
import org.jiemamy.model.DiagramPresentationModel;
import org.jiemamy.model.DiagramPresentations;
import org.jiemamy.model.JiemamyElement;
import org.jiemamy.model.RootModel;
import org.jiemamy.model.connection.ConnectionAdapter;
import org.jiemamy.model.geometory.JmPoint;

/**
 * コネクションにベンドポイントを復元するEDITコマンド。
 * 
 * @author daisuke
 */
public class RevertBendpointsCommand extends AbstractCommand {
	
	private final RootModel rootModel;
	
	/** ダイアグラムエディタのインデックス（エディタ内のタブインデックス） */
	private final int diagramIndex;
	
	/** ベンドポイントが復元されるコネクション */
	private final ConnectionAdapter connectionAdapter;
	
	/** ダイアグラム */
	private final DiagramPresentationModel diagramPresentation;
	
	private final ConnectionProfile connectionProfile;
	
	/** ベンドポイントのリスト */
	private final List<JmPoint> bendpoints;
	

	/**
	 * インスタンスを生成する。
	 * 
	 * @param eventBroker イベント通知用{@link EventBroker}
	 * @param rootModel ルートモデル
	 * @param diagramIndex ダイアグラムエディタのインデックス（エディタ内のタブインデックス）
	 * @param connectionAdapter ベンドポイントが復元されるコネクション
	 * @param points source側からtarget側に向かう順の座標のリスト
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public RevertBendpointsCommand(EventBroker eventBroker, RootModel rootModel, int diagramIndex,
			ConnectionAdapter connectionAdapter, List<JmPoint> points) {
		super(eventBroker);
		Validate.notNull(rootModel);
		Validate.notNull(connectionAdapter);
		Validate.notNull(points);
		this.rootModel = rootModel;
		this.diagramIndex = diagramIndex;
		this.connectionAdapter = connectionAdapter;
		DiagramPresentations diagramPresentations = rootModel.getAdapter(DiagramPresentations.class);
		diagramPresentation = diagramPresentations.get(diagramIndex);
		connectionProfile = diagramPresentation.getConnectionProfiles().get(connectionAdapter);
		bendpoints = connectionProfile.getBendpoints();
	}
	
	public Command getNegateCommand() {
		return new ResetBendpointCommand(getEventBroker(), rootModel, diagramIndex, connectionAdapter);
	}
	
	public JiemamyElement getTarget() {
		return connectionProfile;
	}
	
	@Override
	protected void execute0() {
		bendpoints.clear();
		bendpoints.addAll(bendpoints);
	}
}
