/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/03/16
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.editcommand;

import java.util.Collections;
import java.util.List;

import org.apache.commons.lang.Validate;

import org.jiemamy.EventBroker;
import org.jiemamy.editcommand.Command;
import org.jiemamy.model.JiemamyElement;

/**
 * リスト要素の入れ替えを行うEDITコマンドクラス。
 * 
 * @author daisuke
 */
public class SwapListElementCommand extends AbstractCommand {
	
	/** 操作対象の{@link JiemamyElement} */
	private final JiemamyElement target;
	
	/** 変更対象のリスト */
	private final List<?> list;
	
	/** スワップ主体要素のインデックス */
	private final int subjectIndex;
	
	/** スワップ対象要素のインデックス */
	private final int objectIndex;
	

	/**
	 * インスタンスを生成する。
	 * 
	 * @param eventBroker イベント通知用{@link EventBroker}
	 * @param target 操作対象の{@link JiemamyElement}
	 * @param list 変更対象のリスト
	 * @param subjectIndex スワップ主体要素のインデックス
	 * @param objectIndex スワップ対象要素のインデックス
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public SwapListElementCommand(EventBroker eventBroker, JiemamyElement target, List<?> list, int subjectIndex,
			int objectIndex) {
		super(eventBroker);
		Validate.notNull(target);
		Validate.notNull(list);
		this.target = target;
		this.list = list;
		this.subjectIndex = subjectIndex;
		this.objectIndex = objectIndex;
	}
	
	public Command getNegateCommand() {
		return new SwapListElementCommand(getEventBroker(), target, list, objectIndex, subjectIndex);
	}
	
	/**
	 * 属性を変更する対象のモデルを取得する。
	 * 
	 * @return 属性を変更する対象のモデル
	 */
	public JiemamyElement getTarget() {
		return target;
	}
	
	@Override
	public String toString() {
		return "MoveDownListCommand(" + target + "." + subjectIndex + ")";
	}
	
	@Override
	protected void execute0() {
		Collections.swap(list, subjectIndex, objectIndex);
	}
}
