/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/03/16
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.editcommand;

import java.util.List;

import org.apache.commons.lang.Validate;

import org.jiemamy.EventBroker;
import org.jiemamy.editcommand.Command;
import org.jiemamy.model.JiemamyElement;
import org.jiemamy.model.attribute.ColumnRef;
import org.jiemamy.model.attribute.constraint.ForeignKey;
import org.jiemamy.utils.CollectionsUtil;

/**
 * 外部キーのマッピングをすべて更新するEDITコマンド。
 * 
 * @author daisuke
 */
public class UpdateForeignKeyMappingCommand extends AbstractCommand {
	
	/** 更新対象の外部キー */
	private final ForeignKey foreignKey;
	
	/** 新しいキーカラム列 */
	private final List<ColumnRef> newKeyColumns;
	
	/** 新しい参照カラム列 */
	private final List<ColumnRef> newReferenceColumns;
	
	/** 以前のキーカラム列 */
	private final List<ColumnRef> oldKeyColumns;
	
	/** 以前の参照カラム列 */
	private final List<ColumnRef> oldReferenceColumns;
	

	/**
	 * インスタンスを生成する。
	 * 
	 * @param eventBroker イベント通知用{@link EventBroker}
	 * @param foreignKey 更新対象の外部キー
	 * @param newKeyColumns 新しいキーカラム列
	 * @param newReferenceColumns 新しい参照カラム列
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public UpdateForeignKeyMappingCommand(EventBroker eventBroker, ForeignKey foreignKey,
			List<ColumnRef> newKeyColumns, List<ColumnRef> newReferenceColumns) {
		super(eventBroker);
		Validate.notNull(foreignKey);
		Validate.notNull(newKeyColumns);
		Validate.notNull(newReferenceColumns);
		this.foreignKey = foreignKey;
		this.newKeyColumns = newKeyColumns;
		this.newReferenceColumns = newReferenceColumns;
		oldKeyColumns = CollectionsUtil.newArrayList(foreignKey.getKeyColumns());
		oldReferenceColumns = CollectionsUtil.newArrayList(foreignKey.getReferenceColumns());
	}
	
	public Command getNegateCommand() {
		return new UpdateForeignKeyMappingCommand(getEventBroker(), foreignKey, oldKeyColumns, oldReferenceColumns);
	}
	
	public JiemamyElement getTarget() {
		return foreignKey;
	}
	
	@Override
	protected void execute0() {
		foreignKey.getKeyColumns().clear();
		foreignKey.getKeyColumns().addAll(newKeyColumns);
		foreignKey.getReferenceColumns().clear();
		foreignKey.getReferenceColumns().addAll(newReferenceColumns);
	}
}
