/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/01/20
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.model;

import java.util.List;

import org.apache.commons.lang.Validate;

import org.jiemamy.model.Adapter;
import org.jiemamy.utils.Adaptable;
import org.jiemamy.utils.AdapterNotFoundException;
import org.jiemamy.utils.AnnotationUtil;
import org.jiemamy.utils.CollectionsUtil;

/**
 * {@link Adaptable}インターフェイスの骨格実装。
 * 
 * @author daisuke
 */
public abstract class AbstractAdaptable implements Adaptable {
	
	/** 適用可能なアダプタのリスト */
	private List<Object> adapters = CollectionsUtil.newArrayList();
	

	public <T>T getAdapter(Class<T> adapterClass) {
		assert adapters != null;
		Validate.notNull(adapterClass);
		
		for (Object adapter : adapters) {
			if (adapterClass.isAssignableFrom(adapter.getClass())) {
				return adapterClass.cast(adapter);
			}
		}
		throw new AdapterNotFoundException(adapterClass);
	}
	
	public List<Object> getAdapters() {
		assert adapters != null;
		return CollectionsUtil.newArrayList(adapters);
	}
	
	public boolean hasAdapter(Class<?> adapterClass) {
		assert adapters != null;
		Validate.notNull(adapterClass);
		
		for (Object myAdapter : adapters) {
			if (adapterClass.isAssignableFrom(myAdapter.getClass())) {
				return true;
			}
		}
		return false;
	}
	
	public void registerAdapter(Object adapter) {
		assert adapters != null;
		Validate.notNull(adapter);
		
		Adapter annotation = AnnotationUtil.getTypeAnnotation(adapter, Adapter.class);
		if (annotation == null) {
			throw new IllegalArgumentException(adapter.getClass().getName() + " does not have Adapter annotation.");
		}
		
		adapters.add(adapter);
	}
	
	public <T>T unregisterAdapter(Class<T> adapterClass) {
		T remove = getAdapter(adapterClass);
		adapters.remove(remove);
		return remove;
	}
	
}
