/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2008/06/09
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.model.attribute;

import java.util.List;
import java.util.UUID;

import org.jiemamy.Jiemamy;
import org.jiemamy.exception.ElementNotFoundException;
import org.jiemamy.model.attribute.ColumnModel;
import org.jiemamy.model.attribute.ColumnRef;
import org.jiemamy.model.attribute.constraint.ColumnCheckConstraint;
import org.jiemamy.model.attribute.constraint.NotNullConstraint;
import org.jiemamy.model.attribute.constraint.PrimaryKey;
import org.jiemamy.model.attribute.constraint.UniqueKey;
import org.jiemamy.model.datatype.DataType;
import org.jiemamy.model.entity.TableModel;

/**
 * カラム定義モデル。Artemisにおける{@link ColumnModel}の実装クラス。
 * 
 * @author daisuke
 */
public class ColumnModelImpl extends AbstractAttributeModel implements ColumnModel {
	
	/** 型記述子 */
	private DataType dataType;
	
	/** デフォルト値 */
	private String defaultValue;
	
	private ColumnCheckConstraint checkConstraint;
	
	private NotNullConstraint notNullConstraint;
	
	private PrimaryKey primaryKey;
	
	private UniqueKey uniqueKey;
	

	/**
	 * インスタンスを生成する。
	 * 
	 * @param jiemamy コンテキスト
	 * @param id モデルID
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public ColumnModelImpl(Jiemamy jiemamy, UUID id) {
		super(jiemamy, id);
		setName("");
	}
	
	public boolean checkPrimaryKeyColumn() {
		TableModel tableModel = findDeclaringTable();
		PrimaryKey primaryKey;
		try {
			primaryKey = tableModel.findAttribute(PrimaryKey.class, true);
		} catch (ElementNotFoundException e) {
			return false;
		}
		List<ColumnRef> keyColumns = primaryKey.getKeyColumns();
		for (ColumnRef columnRef : keyColumns) {
			if (columnRef.getReferenceId().equals(getId())) {
				return true;
			}
		}
		return false;
	}
	
	public ColumnCheckConstraint getCheckConstraint() {
		return checkConstraint;
	}
	
	public synchronized DataType getDataType() {
		return dataType;
	}
	
	public String getDefaultValue() {
		return defaultValue;
	}
	
	public NotNullConstraint getNotNullConstraint() {
		return notNullConstraint;
	}
	
	public PrimaryKey getPrimaryKey() {
		return primaryKey;
	}
	
	public UniqueKey getUniqueKey() {
		return uniqueKey;
	}
	
	public void setCheckConstraint(ColumnCheckConstraint checkConstraint) {
		this.checkConstraint = checkConstraint;
	}
	
	public synchronized void setDataType(DataType dataType) {
		this.dataType = dataType;
	}
	
	public void setDefaultValue(String defaultValue) {
		this.defaultValue = defaultValue;
	}
	
	public void setNotNullConstraint(NotNullConstraint notNullConstraint) {
		this.notNullConstraint = notNullConstraint;
	}
	
	public void setPrimaryKey(PrimaryKey primaryKey) {
		this.primaryKey = primaryKey;
	}
	
	public void setUniqueKey(UniqueKey uniqueKey) {
		this.uniqueKey = uniqueKey;
	}
	
	@Override
	public String toString() {
		return "Column " + getName();
//		return ToStringBuilder.reflectionToString(this, ToStringStyle.SHORT_PREFIX_STYLE);
	}
	
}
