/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2008/06/09
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.model.dataset;

import java.util.Map;
import java.util.UUID;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

import org.jiemamy.Jiemamy;
import org.jiemamy.internal.model.AbstractJiemamyElement;
import org.jiemamy.model.attribute.ColumnRef;
import org.jiemamy.model.dataset.RecordModel;

/**
 * レコード（INSERT文1つ分）モデル。
 * 
 * @author daisuke
 */
public class RecordModelImpl extends AbstractJiemamyElement implements RecordModel {
	
	/**
	 * カラムに対応するデータ
	 * 
	 * データは「存在する場合」と「NULLである場合」と「出力されない場合」があり得る。
	 * 「出力されない場合」とは、例えば "FOO" テーブルに BAR, BAZ, QUX カラムが存在する場合に、
	 * INSERT INTO FOO (BAZ, QUX) VALUES('QUUX', 'COURGE');
	 * というINSERT文が発行される場合である。
	 * 
	 * Mapは、この3種類のステートを保持可能である。nullが設定されたkeyも未設定のkeyも、
	 * どちらもnullを返すが、containsKeyメソッドで、両者を判別可能である。
	 * 詳しくは{@link Map#get(Object)}のjavadocを参照のこと。
	 */
	private Map<ColumnRef, String> values;
	

	/**
	 * インスタンスを生成する。
	 * 
	 * @param jiemamy コンテキスト
	 * @param id モデルID
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public RecordModelImpl(Jiemamy jiemamy, UUID id) {
		super(jiemamy, id);
	}
	
	/**
	 * カラムに対応するデータを取得する。
	 * 
	 * @return カラムに対応するデータ
	 */
	public Map<ColumnRef, String> getValues() {
		assert values != null;
		return values;
	}
	
	/**
	 * カラムに対応するデータを設定する。
	 * 
	 * @param values カラムに対応するデータ
	 */
	public void setValues(Map<ColumnRef, String> values) {
		this.values = values;
	}
	
	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this, ToStringStyle.SHORT_PREFIX_STYLE);
	}
}
