/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2008/06/09
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.model.datatype;

import java.util.List;
import java.util.UUID;

import org.jiemamy.Jiemamy;
import org.jiemamy.internal.model.AbstractJiemamyElement;
import org.jiemamy.model.attribute.constraint.ColumnCheckConstraint;
import org.jiemamy.model.attribute.constraint.NotNullConstraint;
import org.jiemamy.model.datatype.BuiltinDataType;
import org.jiemamy.model.datatype.DomainModel;

/**
 * ドメイン定義モデル。
 * 
 * @author daisuke
 */
public class DomainModelImpl extends AbstractJiemamyElement implements DomainModel {
	
	/** ドメイン名 */
	private String name = "";
	
	/** 論理名 */
	private String logicalName;
	
	/** ドメインとして定義された型記述子 */
	private BuiltinDataType dataType;
	
	/** 説明文 */
	private String description;
	
	private ColumnCheckConstraint checkConstraint;
	
	private NotNullConstraint notNullConstraint;
	

	/**
	 * インスタンスを生成する。
	 * 
	 * @param jiemamy コンテキスト
	 * @param id モデルID
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public DomainModelImpl(Jiemamy jiemamy, UUID id) {
		super(jiemamy, id);
	}
	
	@Override
	public <T>T getAdapter(Class<T> adapterClass) {
		if (dataType == null) {
			throw new IllegalStateException("dataType is null.");
		}
		return dataType.getAdapter(adapterClass);
	}
	
	@Override
	public List<Object> getAdapters() {
		if (dataType == null) {
			throw new IllegalStateException("dataType is null.");
		}
		return dataType.getAdapters();
	}
	
	public ColumnCheckConstraint getCheckConstraint() {
		return checkConstraint;
	}
	
	public BuiltinDataType getDataType() {
		return dataType;
	}
	
	public String getDescription() {
		return description;
	}
	
	public String getLogicalName() {
		return logicalName;
	}
	
	public String getName() {
		return name;
	}
	
	public NotNullConstraint getNotNullConstraint() {
		return notNullConstraint;
	}
	
	@Override
	public boolean hasAdapter(Class<?> adapterClass) {
		if (dataType == null) {
			throw new IllegalStateException("dataType is null.");
		}
		return dataType.hasAdapter(adapterClass);
	}
	
	@Override
	public void registerAdapter(Object adapter) {
		if (dataType == null) {
			throw new IllegalStateException("dataType is null.");
		}
		dataType.registerAdapter(adapter);
	}
	
	public void setCheckConstraint(ColumnCheckConstraint check) {
		checkConstraint = check;
	}
	
	public void setDataType(BuiltinDataType dataType) {
		this.dataType = dataType;
	}
	
	public void setDescription(String description) {
		this.description = description;
	}
	
	public void setLogicalName(String logicalName) {
		this.logicalName = logicalName;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public void setNotNullConstraint(NotNullConstraint notNullConstraint) {
		this.notNullConstraint = notNullConstraint;
	}
	
	@Override
	public String toString() {
		return "Domain " + getName();
	}
}
