/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/01/15
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.internal.serializer;

import static org.jiemamy.xml.CoreQName.ADAPTER;
import static org.jiemamy.xml.CoreQName.ATTRIBUTES;
import static org.jiemamy.xml.CoreQName.BEGIN_SCRIPT;
import static org.jiemamy.xml.CoreQName.COLUMN_CHECK_CONSTRAINT;
import static org.jiemamy.xml.CoreQName.COLUMN_PRIMARY_KEY;
import static org.jiemamy.xml.CoreQName.COLUMN_REF;
import static org.jiemamy.xml.CoreQName.COLUMN_REFS;
import static org.jiemamy.xml.CoreQName.COLUMN_UNIQUE_KEY;
import static org.jiemamy.xml.CoreQName.DATASET;
import static org.jiemamy.xml.CoreQName.DATASETS;
import static org.jiemamy.xml.CoreQName.DATA_TYPE;
import static org.jiemamy.xml.CoreQName.DEFAULT_VALUE;
import static org.jiemamy.xml.CoreQName.DEFERRABILITY;
import static org.jiemamy.xml.CoreQName.DEFERRABLE;
import static org.jiemamy.xml.CoreQName.DEFINITION;
import static org.jiemamy.xml.CoreQName.DESCRIPTION;
import static org.jiemamy.xml.CoreQName.DIALECT;
import static org.jiemamy.xml.CoreQName.DOMAIN;
import static org.jiemamy.xml.CoreQName.DOMAINS;
import static org.jiemamy.xml.CoreQName.END_SCRIPT;
import static org.jiemamy.xml.CoreQName.ENTITIES;
import static org.jiemamy.xml.CoreQName.EXPRESSION;
import static org.jiemamy.xml.CoreQName.INDEXES;
import static org.jiemamy.xml.CoreQName.INDEX_COLUMNN;
import static org.jiemamy.xml.CoreQName.INDEX_COLUMNNS;
import static org.jiemamy.xml.CoreQName.INITIALLY_CHECK_TIME;
import static org.jiemamy.xml.CoreQName.LOGICAL_NAME;
import static org.jiemamy.xml.CoreQName.MATCH_TYPE;
import static org.jiemamy.xml.CoreQName.NAME;
import static org.jiemamy.xml.CoreQName.NOT_NULL;
import static org.jiemamy.xml.CoreQName.ON_DELETE;
import static org.jiemamy.xml.CoreQName.ON_UPDATE;
import static org.jiemamy.xml.CoreQName.RECORD;
import static org.jiemamy.xml.CoreQName.REFERENCE_COLUMNS;
import static org.jiemamy.xml.CoreQName.SCHEMA_NAME;
import static org.jiemamy.xml.CoreQName.SORT_ORDER;
import static org.jiemamy.xml.CoreQName.TABLE;
import static org.jiemamy.xml.CoreQName.TABLE_REF;
import static org.jiemamy.xml.CoreQName.TYPE_CATEGORY;
import static org.jiemamy.xml.CoreQName.TYPE_NAME;
import static org.jiemamy.xml.CoreQName.UNIQUE;
import static org.jiemamy.xml.CoreQName.VIEW;

import java.lang.reflect.Field;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.UUID;
import java.util.Map.Entry;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.collections15.list.UnmodifiableList;
import org.apache.commons.lang.BooleanUtils;
import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.Validate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.jiemamy.ReferenceResolver;
import org.jiemamy.Version;
import org.jiemamy.exception.UnexpectedConditionError;
import org.jiemamy.model.Adapter;
import org.jiemamy.model.AdapterType;
import org.jiemamy.model.RootModel;
import org.jiemamy.model.attribute.AttributeModel;
import org.jiemamy.model.attribute.ColumnModel;
import org.jiemamy.model.attribute.ColumnRef;
import org.jiemamy.model.attribute.constraint.ColumnCheckConstraint;
import org.jiemamy.model.attribute.constraint.ColumnConstraint;
import org.jiemamy.model.attribute.constraint.Deferrability;
import org.jiemamy.model.attribute.constraint.ForeignKey;
import org.jiemamy.model.attribute.constraint.KeyConstraint;
import org.jiemamy.model.attribute.constraint.NotNullConstraint;
import org.jiemamy.model.attribute.constraint.PrimaryKey;
import org.jiemamy.model.attribute.constraint.TableCheckConstraint;
import org.jiemamy.model.attribute.constraint.UniqueKey;
import org.jiemamy.model.dataset.DataSetModel;
import org.jiemamy.model.dataset.RecordModel;
import org.jiemamy.model.datatype.BuiltinDataType;
import org.jiemamy.model.datatype.DataType;
import org.jiemamy.model.datatype.DomainModel;
import org.jiemamy.model.datatype.DomainRef;
import org.jiemamy.model.entity.EntityModel;
import org.jiemamy.model.entity.TableModel;
import org.jiemamy.model.entity.TableRef;
import org.jiemamy.model.entity.ViewModel;
import org.jiemamy.model.index.IndexColumnModel;
import org.jiemamy.model.index.IndexModel;
import org.jiemamy.utils.AnnotationUtil;
import org.jiemamy.utils.CollectionsUtil;
import org.jiemamy.utils.DomUtil;
import org.jiemamy.xml.CoreNamespace;
import org.jiemamy.xml.CoreQName;
import org.jiemamy.xml.JiemamyQName;

/**
 * {@link RootModel}からJiemamy XMLモデルのDOMを生成するクラス。
 * 
 * @author daisuke
 */
class JiemamyDomBuilder {
	
	private static Logger logger = LoggerFactory.getLogger(JiemamyDomBuilder.class);
	

	private static Element newColumnConstraintsElement(Element parentElement, ColumnConstraint columnConstraint) {
		if (columnConstraint == null) {
			return null;
		}
		Element element = null;
		if (columnConstraint instanceof NotNullConstraint) {
			NotNullConstraint nnModel = (NotNullConstraint) columnConstraint;
			element = DomUtil.newChild(parentElement, NOT_NULL);
			element.setAttribute(CoreQName.ID.getQNameString(), nnModel.getId().toString());
			DomUtil.newChild(element, NAME, nnModel.getName());
			writeAdapters(element, nnModel.getAdapters());
		} else if (columnConstraint instanceof ColumnCheckConstraint) {
			ColumnCheckConstraint columnCheckModel = (ColumnCheckConstraint) columnConstraint;
			writeColumnCheckConstrait(parentElement, columnCheckModel);
		} else if (columnConstraint instanceof UniqueKey) {
			UniqueKey uniqueKey = (UniqueKey) columnConstraint;
			element = DomUtil.newChild(parentElement, COLUMN_UNIQUE_KEY);
			element.setAttribute(CoreQName.ID.getQNameString(), uniqueKey.getId().toString());
			DomUtil.newChild(element, NAME, uniqueKey.getName());
			writeAdapters(element, uniqueKey.getAdapters());
		} else if (columnConstraint instanceof PrimaryKey) {
			PrimaryKey primaryKey = (PrimaryKey) columnConstraint;
			element = DomUtil.newChild(parentElement, COLUMN_PRIMARY_KEY);
			element.setAttribute(CoreQName.ID.getQNameString(), primaryKey.getId().toString());
			DomUtil.newChild(element, NAME, primaryKey.getName());
			writeAdapters(element, primaryKey.getAdapters());
		}
		return element;
	}
	
	private static Element newDataTypeElement(Element parentElement, DataType dataType) {
		Element dataTypeElement = DomUtil.newChild(parentElement, DATA_TYPE);
		if (dataType instanceof DomainRef) {
			DomainRef domainRef = (DomainRef) dataType;
			dataTypeElement.setAttribute(CoreQName.REF.getQNameString(), domainRef.getReferenceId().toString());
		} else if (dataType instanceof BuiltinDataType) {
			BuiltinDataType builtinDataType = (BuiltinDataType) dataType;
			DomUtil.newChild(dataTypeElement, TYPE_CATEGORY, builtinDataType.getCategory());
			DomUtil.newChild(dataTypeElement, TYPE_NAME, builtinDataType.getTypeName());
			writeAdapters(dataTypeElement, builtinDataType.getAdapters());
		} else if (dataType == null) {
			// nothing to do
		} else {
			throw new UnexpectedConditionError("unknown DataType: " + dataType.getClass().getName());
		}
		return dataTypeElement;
	}
	
	private static SortedSet<Entry<ColumnRef, String>> sortDataEntry(EntityModel entityModel, RecordModel recordModel) {
		SortedSet<Entry<ColumnRef, String>> treeSet =
				new TreeSet<Map.Entry<ColumnRef, String>>(new RecordSetComparator(((TableModel) entityModel)
					.findColumns()));
		for (Map.Entry<ColumnRef, String> entry2 : recordModel.getValues().entrySet()) {
			treeSet.add(entry2);
		}
		return treeSet;
	}
	
	private static void writeAdapters(Element parentElement, List<Object> adapters) {
		for (Object adapter : adapters) {
			Adapter annotation = AnnotationUtil.getTypeAnnotation(adapter, Adapter.class);
			assert annotation != null;
			if (annotation.value() == AdapterType.MODEL) {
				Element adapterElement = DomUtil.newChild(parentElement, ADAPTER);
				adapterElement.setAttribute(CoreQName.CLASS.getQNameString(), adapter.getClass().getName());
				boolean existSpecificValue = false;
				for (Field field : adapter.getClass().getDeclaredFields()) {
					String value = null;
					try {
						field.setAccessible(true);
						value = ObjectUtils.toString(field.get(adapter), null);
					} catch (IllegalArgumentException e) {
						logger.error(e.getMessage());
						throw new UnexpectedConditionError("フィールドは必ず存在するはず", e);
					} catch (IllegalAccessException e) {
						logger.error(e.getMessage());
						throw new UnexpectedConditionError("setAccessible(true)しているのでアクセスできるはず", e);
					}
					if (value != null) {
						DomUtil.newChild(adapterElement, new QName(CoreNamespace.NS_CORE.getNamespaceURI().toString(),
								field.getName()), value);
						existSpecificValue = true;
					}
				}
				if (existSpecificValue == false) {
					parentElement.removeChild(adapterElement);
				}
			}
		}
	}
	
	private static void writeColumnCheckConstrait(Element parentElement, ColumnCheckConstraint checkConstraint) {
		Element checkElement = DomUtil.newChild(parentElement, COLUMN_CHECK_CONSTRAINT);
		checkElement.setAttribute(CoreQName.ID.getQNameString(), checkConstraint.getId().toString());
		DomUtil.newChild(checkElement, NAME, checkConstraint.getName());
		DomUtil.newChild(checkElement, EXPRESSION, checkConstraint.getExpression());
		writeAdapters(checkElement, checkConstraint.getAdapters());
	}
	

	private RootModel rootModel;
	

	/**
	 * インスタンスを生成する。
	 * 
	 * @param rootModel 読み込む{@link RootModel}
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public JiemamyDomBuilder(RootModel rootModel) {
		Validate.notNull(rootModel);
		
		this.rootModel = rootModel;
	}
	
	/**
	 * DOM Documentを構築する。
	 * 
	 * @return DOM Document
	 * @throws ParserConfigurationException DocumentBuilderFactoryの設定異常
	 */
	public Document build() throws ParserConfigurationException {
		DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
		documentBuilderFactory.setNamespaceAware(true);
		DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
		Document document = documentBuilder.newDocument();
		
		Element rootElement = DomUtil.newElement(document, CoreQName.JIEMAMY);
		rootElement.setAttribute(CoreQName.ID.getQNameString(), rootModel.getId().toString());
		rootElement.setAttribute(CoreQName.VERSION.getQNameString(), Version.SPEC);
		rootElement.setAttribute("xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
		
		StringBuilder sb = new StringBuilder();
		sb.append(CoreNamespace.NS_CORE.getNamespaceURI().toString());
		sb.append(" ");
		sb.append(CoreNamespace.NS_CORE.getXmlSchemaLocation());
		rootElement.setAttribute("xsi:schemaLocation", sb.toString());
		
		document.appendChild(rootElement);
		
		buildMiscSection(rootElement);
		buildDomainSection(rootElement);
		buildNodeSection(rootElement);
		buildDataSetSection(rootElement);
		
		return document;
	}
	
	private void buildColumn(Element attributeElement, ColumnModel columnModel) {
		newDataTypeElement(attributeElement, columnModel.getDataType());
		DomUtil.newChild(attributeElement, DEFAULT_VALUE, columnModel.getDefaultValue());
		newColumnConstraintsElement(attributeElement, columnModel.getNotNullConstraint());
		newColumnConstraintsElement(attributeElement, columnModel.getUniqueKey());
		newColumnConstraintsElement(attributeElement, columnModel.getPrimaryKey());
		newColumnConstraintsElement(attributeElement, columnModel.getCheckConstraint());
	}
	
	private void buildDataSetSection(Element rootElement) {
		Element insertDataSetElement = DomUtil.newChild(rootElement, DATASETS);
		for (DataSetModel insertDataSetModel : rootModel.getDataSets()) {
			Element insertDataSet = DomUtil.newChild(insertDataSetElement, DATASET);
			insertDataSet.setAttribute(CoreQName.ID.getQNameString(), insertDataSetModel.getId().toString());
			DomUtil.newChild(insertDataSet, NAME, insertDataSetModel.getName());
			for (Map.Entry<TableRef, List<RecordModel>> entry : insertDataSetModel.getRecords().entrySet()) {
				TableRef tableRef = entry.getKey();
				ReferenceResolver resolver = insertDataSetModel.getJiemamy().getReferenceResolver();
				EntityModel resolved = resolver.resolve(tableRef);
				Element entityElement = DomUtil.newChild(insertDataSet, TABLE_REF);
				entityElement.setAttribute(CoreQName.REF.getQNameString(), tableRef.getReferenceId().toString());
				for (RecordModel recordModel : entry.getValue()) {
					Element recordElement = DomUtil.newChild(entityElement, RECORD);
					recordElement.setAttribute(CoreQName.ID.getQNameString(), recordModel.getId().toString());
					SortedSet<Entry<ColumnRef, String>> treeSet = sortDataEntry(resolved, recordModel);
					for (Map.Entry<ColumnRef, String> entry2 : treeSet) {
						String value = entry2.getValue();
						Element columnDataElement;
						columnDataElement = DomUtil.newChild(recordElement, COLUMN_REF, value);
						if (columnDataElement != null) {
							columnDataElement.setAttribute(CoreQName.REF.getQNameString(), entry2.getKey()
								.getReferenceId().toString());
						}
					}
				}
			}
		}
	}
	
	private void buildDomainSection(Element rootElement) {
		Element domainsElement = DomUtil.newChild(rootElement, DOMAINS);
		for (DomainModel domainModel : rootModel.getDomains()) {
			Element domainElement = DomUtil.newChild(domainsElement, DOMAIN);
			domainElement.setAttribute(CoreQName.ID.getQNameString(), domainModel.getId().toString());
			DomUtil.newChild(domainElement, NAME, domainModel.getName());
			DomUtil.newChild(domainElement, LOGICAL_NAME, domainModel.getLogicalName());
			DomUtil.newChild(domainElement, DESCRIPTION, domainModel.getDescription());
			newDataTypeElement(domainElement, domainModel.getDataType());
			
			NotNullConstraint notNull = domainModel.getNotNullConstraint();
			if (notNull != null) {
				Element notNullElement = DomUtil.newChild(domainElement, NOT_NULL);
				notNullElement.setAttribute(CoreQName.ID.getQNameString(), notNull.getId().toString());
				DomUtil.newChild(notNullElement, NAME, notNull.getName());
				writeAdapters(notNullElement, notNull.getAdapters());
			}
			
			ColumnCheckConstraint check = domainModel.getCheckConstraint();
			if (check != null) {
				writeColumnCheckConstrait(domainElement, check);
			}
		}
	}
	
	private void buildKeyConstraint(Element attributeElement, KeyConstraint keyeyConstraint) {
		Element columnRefsElement = DomUtil.newChild(attributeElement, COLUMN_REFS);
		for (ColumnRef pkColumn : keyeyConstraint.getKeyColumns()) {
			Element columnRefElement = DomUtil.newChild(columnRefsElement, COLUMN_REF);
			columnRefElement.setAttribute(CoreQName.REF.getQNameString(), pkColumn.getReferenceId().toString());
		}
		
		if (keyeyConstraint instanceof ForeignKey) {
			ForeignKey foreignKey = (ForeignKey) keyeyConstraint;
			
			Element kyColumnsElement = DomUtil.newChild(attributeElement, REFERENCE_COLUMNS);
			for (ColumnRef keyColumnRef : foreignKey.getReferenceColumns()) {
				Element columnRefElement = DomUtil.newChild(kyColumnsElement, COLUMN_REF);
				columnRefElement.setAttribute(CoreQName.REF.getQNameString(), keyColumnRef.getReferenceId().toString());
			}
			
			if (foreignKey.getDeferrability() != null) {
				Deferrability deferrability = foreignKey.getDeferrability();
				Element deferrabilityElement = DomUtil.newChild(attributeElement, DEFERRABILITY);
				deferrabilityElement.setAttribute(CoreQName.ID.getQNameString(), deferrability.getId().toString());
				DomUtil.newChild(deferrabilityElement, DEFERRABLE, BooleanUtils.toStringTrueFalse(deferrability
					.isDeferrable()));
				DomUtil.newChild(deferrabilityElement, INITIALLY_CHECK_TIME, deferrability.getInitiallyCheckTime());
			}
			
			DomUtil.newChild(attributeElement, MATCH_TYPE, foreignKey.getMatchType());
			DomUtil.newChild(attributeElement, ON_DELETE, foreignKey.getOnDelete());
			DomUtil.newChild(attributeElement, ON_UPDATE, foreignKey.getOnUpdate());
		}
	}
	
	private void buildMiscSection(Element rootElement) {
		DomUtil.newChild(rootElement, DIALECT, rootModel.getDialectClassName());
		DomUtil.newChild(rootElement, SCHEMA_NAME, rootModel.getSchemaName());
		DomUtil.newChild(rootElement, DESCRIPTION, rootModel.getDescription());
		DomUtil.newChild(rootElement, BEGIN_SCRIPT, rootModel.getBeginScript());
		DomUtil.newChild(rootElement, END_SCRIPT, rootModel.getEndScript());
	}
	
	private void buildNodeSection(Element rootElement) {
		Element entitiesElement = DomUtil.newChild(rootElement, ENTITIES);
		for (EntityModel entityModel : rootModel.getEntities()) {
			Element entityElement = null;
			
			if (entityModel instanceof TableModel) {
				TableModel tableModel = (TableModel) entityModel;
				entityElement = buildTable(entitiesElement, tableModel);
			} else if (entityModel instanceof ViewModel) {
				ViewModel viewModel = (ViewModel) entityModel;
				entityElement = buildView(entitiesElement, viewModel);
			} else {
				throw new UnexpectedConditionError("unknown entity:" + entityModel.getClass().getName());
			}
			writeAdapters(entityElement, entityModel.getAdapters());
			entityElement.setAttribute(CoreQName.ID.getQNameString(), entityModel.getId().toString());
			entitiesElement.appendChild(entityElement);
		}
	}
	
	/**
	 * テーブル要素を構築する。
	 * 
	 * @param entitiesElement 親となるentities要素
	 * @param tableModel 対象{@link TableModel}
	 * @return 構築したXML要素
	 */
	private Element buildTable(Element entitiesElement, TableModel tableModel) {
		Element entityElement;
		entityElement = DomUtil.newChild(entitiesElement, TABLE);
		DomUtil.newChild(entityElement, NAME, tableModel.getName());
		DomUtil.newChild(entityElement, LOGICAL_NAME, tableModel.getLogicalName());
		DomUtil.newChild(entityElement, DESCRIPTION, tableModel.getDescription());
		DomUtil.newChild(entityElement, BEGIN_SCRIPT, tableModel.getBeginScript());
		DomUtil.newChild(entityElement, END_SCRIPT, tableModel.getEndScript());
		
		Element attributesElement = DomUtil.newChild(entityElement, ATTRIBUTES);
		for (AttributeModel attributeModel : tableModel.getAttributes()) {
			Element attributeElement = DomUtil.newChild(attributesElement, getQName(attributeModel));
			attributeElement.setAttribute(CoreQName.ID.getQNameString(), attributeModel.getId().toString());
			DomUtil.newChild(attributeElement, NAME, attributeModel.getName());
			DomUtil.newChild(attributeElement, LOGICAL_NAME, attributeModel.getLogicalName());
			DomUtil.newChild(attributeElement, DESCRIPTION, attributeModel.getDescription());
			if (attributeModel instanceof ColumnModel) {
				ColumnModel columnModel = (ColumnModel) attributeModel;
				buildColumn(attributeElement, columnModel);
			} else if (attributeModel instanceof KeyConstraint) {
				KeyConstraint keyeyConstraint = (KeyConstraint) attributeModel;
				buildKeyConstraint(attributeElement, keyeyConstraint);
			} else if (attributeModel instanceof TableCheckConstraint) {
				TableCheckConstraint tableCheckConstraint = (TableCheckConstraint) attributeModel;
				DomUtil.newChild(attributeElement, EXPRESSION, tableCheckConstraint.getExpression());
				writeAdapters(attributeElement, tableCheckConstraint.getAdapters());
			}
			writeAdapters(attributeElement, attributeModel.getAdapters());
		}
		
		Element indexesElement = DomUtil.newChild(entityElement, INDEXES);
		for (IndexModel indexModel : tableModel.getIndexes()) {
			Element indexElement = DomUtil.newChild(indexesElement, CoreQName.INDEX);
			indexElement.setAttribute(CoreQName.ID.getQNameString(), indexModel.getId().toString());
			DomUtil.newChild(indexElement, NAME, indexModel.getName());
			
			Element indexColumnsElement = DomUtil.newChild(indexElement, INDEX_COLUMNNS);
			for (IndexColumnModel indexColumnModel : indexModel.getIndexColumns()) {
				Element indexColumnElement = DomUtil.newChild(indexColumnsElement, INDEX_COLUMNN);
				indexColumnElement.setAttribute(CoreQName.ID.getQNameString(), indexColumnModel.getId().toString());
				Element indexColumnRefElement = DomUtil.newChild(indexColumnElement, COLUMN_REF);
				indexColumnRefElement.setAttribute(CoreQName.REF.getQNameString(), indexColumnModel.getColumnRef()
					.getReferenceId().toString());
				DomUtil.newChild(indexColumnElement, SORT_ORDER, indexColumnModel.getSortOrder());
			}
			
			Element uniqueElement = DomUtil.newChild(indexElement, UNIQUE);
			uniqueElement.setTextContent(Boolean.toString(indexModel.isUnique()));
		}
		return entityElement;
	}
	
	/**
	 * ビュー要素を構築する。
	 * 
	 * @param entitiesElement 親となるentities要素
	 * @param viewModel 対象{@link ViewModel}
	 * @return 構築したXML要素
	 */
	private Element buildView(Element entitiesElement, ViewModel viewModel) {
		Element entityElement;
		entityElement = DomUtil.newChild(entitiesElement, VIEW);
		DomUtil.newChild(entityElement, NAME, viewModel.getName());
		DomUtil.newChild(entityElement, LOGICAL_NAME, viewModel.getLogicalName());
		DomUtil.newChild(entityElement, DESCRIPTION, viewModel.getDescription());
		DomUtil.newChild(entityElement, BEGIN_SCRIPT, viewModel.getBeginScript());
		DomUtil.newChild(entityElement, END_SCRIPT, viewModel.getEndScript());
		DomUtil.newChild(entityElement, DEFINITION, viewModel.getDefinition());
		return entityElement;
	}
	
	private JiemamyQName getQName(AttributeModel attributeModel) {
		if (attributeModel instanceof ColumnModel) {
			return CoreQName.COLUMN;
		} else if (attributeModel instanceof PrimaryKey) {
			return CoreQName.PRIMARY_KEY;
		} else if (attributeModel instanceof UniqueKey) {
			return CoreQName.UNIQUE_KEY;
		} else if (attributeModel instanceof ForeignKey) {
			return CoreQName.FOREIGN_KEY;
		} else if (attributeModel instanceof TableCheckConstraint) {
			return CoreQName.TABLE_CHECK_CONSTRAINT;
		} else {
			return null;
		}
	}
	

	/**
	 * 出力レコードの並び順を、カラム定義順に合わせるコンパレータ。
	 * 
	 * @author daisuke
	 */
	private static class RecordSetComparator implements Comparator<Entry<ColumnRef, String>> {
		
		private final List<UUID> order;
		

		RecordSetComparator(List<ColumnModel> columns) {
			List<UUID> tmpOrder = CollectionsUtil.newArrayList();
			for (ColumnModel columnModel : columns) {
				tmpOrder.add(columnModel.getId());
			}
			order = UnmodifiableList.decorate(tmpOrder);
		}
		
		public int compare(Entry<ColumnRef, String> o1, Entry<ColumnRef, String> o2) {
			int i1 = order.indexOf(o1.getKey().getReferenceId());
			int i2 = order.indexOf(o2.getKey().getReferenceId());
			return i1 - i2;
		}
		
	}
}
