/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/01/26
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.model.attribute;

import org.jiemamy.model.Adapter;
import org.jiemamy.model.AdapterType;

/**
 * 代表カラムかどうかを表すアラプタ。
 * 
 * <p>例えばEMP, DEPTテーブルがあり、EMPがDEPTを外部キー参照しているとする。
 * EMPのINSERT文用データを入力中に、ユーザは「DEPT_ID = 3 の部署」ではなく「経理部」をイメージするのが自然である。
 * SQL的には 3 を出力すべきだが、入力・選択は「経理部」で行いたい。このように、DEPTテーブルを代表するカラムを代表カラムと呼ぶ。</p>
 * 
 * @author daisuke
 */
@Adapter(AdapterType.MODEL)
public class RepresentationAdapter {
	
	private Boolean representation;
	

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		RepresentationAdapter other = (RepresentationAdapter) obj;
		if (representation == null) {
			if (other.representation != null) {
				return false;
			}
		} else if (representation.equals(other.representation) == false) {
			return false;
		}
		return true;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((representation == null) ? 0 : representation.hashCode());
		return result;
	}
	
	/**
	 * 代表カラムかどうかを取得する。
	 * @return 代表カラムかどうか
	 */
	public Boolean isRepresentation() {
		return representation;
	}
	
	/**
	 * 代表カラムかどうかを設定する。
	 * @param representation 代表カラムかどうか
	 */
	public void setRepresentation(Boolean representation) {
		this.representation = representation;
	}
	
	@Override
	public String toString() {
		return "RepresentationAdapter[" + representation + "]";
	}
}
