/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2008/07/20
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.model.datatype.adapter;

import org.jiemamy.model.Adapter;
import org.jiemamy.model.AdapterType;
import org.jiemamy.model.datatype.DataTypeAdapter;

/**
 * サイズを持つDataType用のアダプタ。
 * @author daisuke
 */
@Adapter(AdapterType.MODEL)
public class SizedDataTypeAdapter implements DataTypeAdapter {
	
	/** サイズ */
	private Integer size;
	

	public void copy(DataTypeAdapter src) {
		if (src == null) {
			throw new IllegalArgumentException("The validated object is null");
		}
		if (src instanceof SizedDataTypeAdapter) {
			size = ((SizedDataTypeAdapter) src).size;
		} else {
			throw new IllegalArgumentException(src.getClass().toString() + " (expected is " + this.getClass().getName()
					+ ")");
		}
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final SizedDataTypeAdapter other = (SizedDataTypeAdapter) obj;
		if (size == null) {
			if (other.size != null) {
				return false;
			}
		} else if (size.equals(other.size) == false) {
			return false;
		}
		return true;
	}
	
	/**
	 * サイズを取得する。
	 * 
	 * @return サイズ
	 */
	public Integer getSize() {
		return size;
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((size == null) ? 0 : size.hashCode());
		return result;
	}
	
	/**
	 * サイズを設定する。
	 * 
	 * @param size サイズ
	 */
	public void setSize(Integer size) {
		this.size = size;
	}
	
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		if (size != null) {
			sb.append("(");
			sb.append(size);
			sb.append(")");
		}
		return sb.toString();
	}
}
