/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/01/19
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.utils.model;

import org.apache.commons.lang.Validate;

import org.jiemamy.ReferenceResolver;
import org.jiemamy.exception.UnexpectedConditionError;
import org.jiemamy.model.RootModel;
import org.jiemamy.model.attribute.AttributeModel;
import org.jiemamy.model.attribute.ColumnModel;
import org.jiemamy.model.attribute.ColumnRef;
import org.jiemamy.model.attribute.constraint.ForeignKey;
import org.jiemamy.model.attribute.constraint.PrimaryKey;
import org.jiemamy.model.attribute.constraint.UniqueKey;
import org.jiemamy.model.entity.TableModel;

/**
 * {@link AttributeModel}のユーティリティクラス。
 * 
 * @author daisuke
 */
public final class AttributeUtil {
	
	/**
	 * 自動的に属性名を生成し、設定する。
	 * 
	 * @param attributeModel 設定対象の属性
	 * @param rootModel ルートモデル
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public static void autoDenominate(AttributeModel attributeModel, RootModel rootModel) {
		Validate.notNull(attributeModel);
		Validate.notNull(rootModel);
		ReferenceResolver resolver = rootModel.getJiemamy().getReferenceResolver();
		
		TableModel tableModel = attributeModel.findDeclaringTable();
		
		StringBuilder sb = new StringBuilder();
		if (attributeModel instanceof ColumnModel) {
			sb.append("column_");
		} else if (attributeModel instanceof PrimaryKey) {
			sb.append(tableModel.getName());
			sb.append("_pkey");
		} else if (attributeModel instanceof UniqueKey) {
			UniqueKey uniqueKey = (UniqueKey) attributeModel;
			sb.append(tableModel.getName());
			sb.append("_");
			for (ColumnRef columnRef : uniqueKey.getKeyColumns()) {
				ColumnModel columnModel = resolver.resolve(columnRef);
				sb.append(columnModel.getName());
				sb.append("_");
			}
			sb.append("ukey");
		} else if (attributeModel instanceof ForeignKey) {
			ForeignKey foreignKey = (ForeignKey) attributeModel;
			sb.append(tableModel.getName());
			sb.append("_");
			for (ColumnRef columnRef : foreignKey.getKeyColumns()) {
				ColumnModel columnModel = resolver.resolve(columnRef);
				sb.append(columnModel.getName());
				sb.append("_");
			}
			sb.append("fkey");
		}
		
		attributeModel.setName(sb.toString());
	}
	
	private AttributeUtil() {
		throw new UnexpectedConditionError("不到達ポイント");
	}
}
