/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/01/19
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.utils.model;

import org.apache.commons.lang.Validate;

import org.jiemamy.JiemamyFactory;
import org.jiemamy.exception.UnexpectedConditionError;
import org.jiemamy.model.attribute.ColumnModel;
import org.jiemamy.model.datatype.DomainModel;
import org.jiemamy.model.datatype.DomainRef;

/**
 * {@link ColumnModel}のユーティリティクラス。
 * 
 * @author daisuke
 */
public final class ColumnUtil {
	
	/**
	 * カラムにドメイン型をデータ型として設定する。
	 * 
	 * @param columnModel 対象カラム
	 * @param domainModel 設定するドメイン型
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public static void setDataType(ColumnModel columnModel, DomainModel domainModel) {
		Validate.notNull(columnModel);
		Validate.notNull(domainModel);
		
		JiemamyFactory factory = domainModel.getJiemamy().getFactory();
		DomainRef ref = factory.newReference(domainModel);
		columnModel.setDataType(ref);
	}
	
	private ColumnUtil() {
		throw new UnexpectedConditionError("不到達ポイント");
	}
}
