/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/01/19
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.utils.model;

import org.jiemamy.exception.ElementNotFoundException;
import org.jiemamy.exception.TooManyElementsException;
import org.jiemamy.exception.UnexpectedConditionError;
import org.jiemamy.model.RootModel;
import org.jiemamy.model.entity.EntityModel;
import org.jiemamy.model.entity.TableModel;
import org.jiemamy.model.entity.ViewModel;

/**
 * {@link EntityModel}のユーティリティクラス。
 * 
 * @author daisuke
 */
public final class EntityUtil {
	
	/**
	 * 自動的にエンティティ名を生成し、設定する。
	 * 
	 * @param entityModel 設定対象のエンティティ
	 * @param rootModel ルートモデル
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public static void autoDenominate(EntityModel entityModel, RootModel rootModel) {
		StringBuilder sb;
		int counter = 0;
		boolean duplicated;
		do {
			sb = new StringBuilder();
			Class<? extends EntityModel> target;
			if (entityModel instanceof TableModel) {
				sb.append("TABLE_");
				target = TableModel.class;
			} else if (entityModel instanceof ViewModel) {
				sb.append("VIEW_");
				target = ViewModel.class;
			} else {
				throw new UnexpectedConditionError("Unknown entity: " + entityModel.getClass().toString());
			}
			sb.append(++counter);
			try {
				rootModel.getEntity(target, sb.toString());
				duplicated = true;
			} catch (TooManyElementsException e) {
				duplicated = true;
			} catch (ElementNotFoundException e) {
				duplicated = false;
			}
		} while (duplicated);
		
		entityModel.setName(sb.toString());
	}
	
	private EntityUtil() {
		throw new UnexpectedConditionError("不到達ポイント");
	}
	
}
