/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/02/04
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.utils.model;

import org.apache.commons.lang.Validate;

import org.jiemamy.JiemamyFactory;
import org.jiemamy.model.DiagramPresentationModel;
import org.jiemamy.model.NodeProfile;
import org.jiemamy.model.connection.ConnectionAdapter;
import org.jiemamy.model.geometory.JmRectangle;
import org.jiemamy.model.node.NodeAdapter;

/**
 * {@link DiagramPresentationModel}のユーティリティクラス。
 * 
 * @author daisuke
 */
public class PresentationUtil {
	
	/**
	 * ベンドポイントをリセットする。
	 * 
	 * @param diagramPresentation 対象ダイアグラム
	 * @param connectionAdapter リセット対象のコネクション
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public static void resetBenpoinsts(DiagramPresentationModel diagramPresentation, ConnectionAdapter connectionAdapter) {
		Validate.notNull(diagramPresentation);
		Validate.notNull(connectionAdapter);
		diagramPresentation.getConnectionProfiles().get(connectionAdapter).getBendpoints().clear();
	}
	
	/**
	 * 位置サイズ情報を設定する。
	 * 
	 * @param presentation 対象ダイアグラム
	 * @param nodeAdapter ノード
	 * @param boundary 位置サイズ情報
	 * @throws IllegalArgumentException 引数に{@code null}を与えた場合
	 */
	public static void setBoundary(DiagramPresentationModel presentation, NodeAdapter nodeAdapter, JmRectangle boundary) {
		Validate.notNull(presentation);
		Validate.notNull(nodeAdapter);
		Validate.notNull(boundary);
		NodeProfile nodeProfile = presentation.getNodeProfiles().get(nodeAdapter);
		if (nodeProfile == null) {
			JiemamyFactory factory = presentation.getJiemamy().getFactory();
			nodeProfile = factory.newModel(NodeProfile.class);
			presentation.getNodeProfiles().put(nodeAdapter, nodeProfile);
		}
		nodeProfile.setBoundary(boundary);
		
	}
	
	private PresentationUtil() {
	}
}
