/*
 * Copyright 2007-2009 Jiemamy Project and the Others.
 * Created on 2009/01/21
 *
 * This file is part of Jiemamy.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.jiemamy.validator.impl;

import java.util.Locale;

import org.apache.commons.lang.ArrayUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.jiemamy.validator.AbstractProblem;
import org.jiemamy.validator.Problem;

/**
 * 同じインスタンスへの参照を複数保持していないかどうかを調べるバリデータ。
 * 
 * <ul>
 *   <li>全てのモデルは、同じモデルのインスタンスを複数箇所から参照してはならない。</li>
 * </uL>
 * 
 * @author daisuke
 */
public class InstanceCollisionValidator extends AbstractTraversalValidator<Integer> {
	
	private static Logger logger = LoggerFactory.getLogger(InstanceCollisionValidator.class);
	

	@Override
	void check(Object element, String pos) {
		if (element == null) {
			return;
		}
		int identityHashCode = System.identityHashCode(element);
		if (ids.containsKey(identityHashCode)) {
			result.add(new InstanceCollisionProblem(identityHashCode, ids.get(identityHashCode), pos));
			logger.warn("collision : " + ArrayUtils.toString(new RuntimeException().getStackTrace()));
		}
		ids.put(identityHashCode, pos);
	}
	

	/**
	 * ID名が衝突していることを示す{@link Problem}オブジェクト。
	 * 
	 * @author daisuke
	 */
	static class InstanceCollisionProblem extends AbstractProblem {
		
		/**
		 * インスタンスを生成する。
		 * 
		 * @param identityHashCode 衝突したID
		 * @param prev 前回見つかった位置を示す文字列
		 * @param current 現在位置を表す文字列
		 */
		public InstanceCollisionProblem(int identityHashCode, String prev, String current) {
			super("F0020");
			registerMessage(Locale.JAPAN, "{1}と{2}においてインスタンス \"{0}\" が重複しています");
			registerMessage(Locale.US, "Duplicate instance {0} of {1} and {2}");
			setArguments(new Object[] {
				identityHashCode,
				prev,
				current
			});
		}
		
		public Severity getSeverity() {
			return Severity.FATAL;
		}
		
	}
	
}
